# import zq_tts_pyttsx3 as tts
# tts.tts_pyttsx3_speek('66666')

# import zq_tts_win32com as tts
# tts.tts_win32com_speek('66666')

# import zq_tts_tencent as tts
# tts.tts_tencent_class().tts_tencent_req(tts.tts_tencent_data_request_class(save='./tencent_tts.wav', text='9999哈哈哈'))

# import zq_tts_youdao as tts
# tts.tts_youdao_web_class().tts_youdao_req(save='./youdao_tts.wav', text='9999哈哈哈')